#include "manual_slm.h"
#include "manual_slm_keys.h"
#include "slm_driver.h"
#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <curses.h>
#include <g++-3/iostream.h>
#include <stdlib.h>



// new fixed manual_slm- takes the size of the slm to be the undistorted height
// the distroted size to be horizontal and 1.1 times less big (this is the one that is squash after all)

main(int argc,char ** argv){

// set initial values

float xoff=0;
float focal_length=2e-3;
float wavelength=37.5e-6;
float slm_height=1.76e-3;
float distortion_factor=1.1;
float msr=0.5;
int frame_rate=1;

float step_xoff=100e-6;
float step_focal_length=0.1e-3;
float step_wavelength=1e-6;

int quit=FALSE;
int update_slm=FALSE;
int first_run=TRUE;
char keypress;
char kp2;
char inptvalue[12];
float px,py;

	if(argc!=1){
		cout<<"No arguments!\n";
		exit(3);
		}
	initscr();
	cbreak();
	noecho();
	echo();
	noecho();
	nonl();
	intrflush(stdscr, FALSE);
	refresh();

cout<<"CAUTION: This program uses matt's SLM_upload arcs\r\n";
cout<<"Internally this function calls the compress direction X\r\n";
cout<<"and the uncompressed direction is called Y\r\n";
cout<<"As of June 2001 X corresponds to vertical (+ve goes down on sample)\r\n";
cout<<"and Y corresponds to horizontal (+ve goes right on sample)\r\n";
cout<<"The Y direction is squished by the optical beamsteer tilt\r\n";
cout<<"manual_slm uses (x,y)->(xoff,F)->(Y,X) hence +F goes down\r\n";
cout<<"+ve xoffset goes right\r\n\n";
	cout<<"Matt / Steve's fixed manual arc manouverer.\r\n"<<flush;
	cout<<"Press `h' for help, or `q' for quit, `a' to show the arc attributes.\r\n"<<flush;
	SLM_reset();
	SLM_repeat(25000);


	// now work out the conversion factors from real world units into SLM pixels


	do{
		keypress=getchar();
		switch(keypress){
			case LINE_SPACING_PLUS:
				wavelength+=step_wavelength;
				cout<<"wavelength: "<<wavelength*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case LINE_SPACING_MINUS:
				wavelength-=step_wavelength;
				cout<<"wavelength: "<<wavelength*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case FOCAL_LENGTH_PLUS:
				focal_length+=step_focal_length;
				cout<<"Mean focal length: "<<focal_length*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case FOCAL_LENGTH_MINUS:
				focal_length-=step_focal_length;
				cout<<"Mean focal length: "<<focal_length*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case X_OFFSET_PLUS:
				xoff+=step_xoff;
				cout<<"x offset: "<<xoff*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case X_OFFSET_MINUS:
				xoff-=step_xoff;
				cout<<"x offset: "<<xoff*1e6<<" microns\r"<<endl;
				update_slm=TRUE;
				break;


			case SET_TO:
				cout<<"Set "<<flush;
				kp2=getchar();
				switch(kp2){
					case LINE_SPACING: //kept for backwards 'human compatability'
						cout<<"wavelength.\r\nCurrent wavelength: "<<wavelength*1e6<<" microns. Enter new wavelength: "<<flush;
						echo();
						getnstr(inptvalue,10);wavelength=strtod(inptvalue,NULL)/1e6;
						noecho();
						break;
					case WAVELENGTH: //exactly the same thing as above, just a different keypress
						cout<<"wavelength.\r\nCurrent wavelength: "<<wavelength*1e6<<" microns. Enter new wavelength: "<<flush;
						echo();
						getnstr(inptvalue,10);wavelength=strtod(inptvalue,NULL)/1e6;
						noecho();
						break;
					case FOCAL_LENGTH:
						cout<<"mean focal length.\r\nCurrent mean focal length: "<<focal_length*1e6<<" microns . Enter new mean focal length: "<<flush;
						echo();
						getnstr(inptvalue,10);focal_length=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					case X_OFFSET:
						cout<<"x offset.\r\nCurrent x offset: "<<xoff*1e6<<" microns. Enter new x offset: "<<flush;
						echo();
						getnstr(inptvalue,10);xoff=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					case SLM_SIZE:
						cout<<"image size.\r\nCurrent image size: "<<slm_height*1e6<<" microns. Enter new image size: "<<flush;
						echo();
						getnstr(inptvalue,10);slm_height=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					case DISTORTION_FACTOR:
						cout<<"distortion factor.\r\nCurrent distortion factor: "<<distortion_factor<<". Enter distortion factor: "<<flush;
						echo();
						getnstr(inptvalue,10);distortion_factor=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						break;
					case REP_RATE:
						cout<<"frame repetition rate.\r\nCurrent frame repetition rate: "<<frame_rate<<" Hz. Enter new repetition rate (integer): "<<flush;
						echo();
						getnstr(inptvalue,10);frame_rate=strtol(inptvalue,NULL,10);
						cout<<"\r"<<flush;
						noecho();
						update_slm=TRUE;
						break;
					case MS_RATIO:
						cout<<"mark:space ratio.\r\nCurrent mark:space ratio: "<<msr<<" Enter new mark:space ratio: "<<flush;
						echo();
						getnstr(inptvalue,10);msr=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						update_slm=TRUE;
						break;
					default:
						cout<<"Nothing changed.\n";
						cout<<"\r"<<endl;
						break;
					}
				update_slm=TRUE;
				break;

			case FACTOR:
				cout<<"Set factor: "<<flush;
				kp2=getchar();
				switch(kp2){
					case LINE_SPACING: //kept for backwards 'human compatability'
						cout<<"wavelength.\r\nCurrent wavelength factor: "<<step_wavelength*1e6<<" microns. Enter new wavelength factor: "<<flush;
						echo();
						getnstr(inptvalue,10);step_wavelength=strtod(inptvalue,NULL)/1e6;
						noecho();
						cout<<"\r"<<flush;
						break;
					case WAVELENGTH: //exactly the same thing as above, just a different keypress
						cout<<"wavelength.\r\nCurrent wavelength factor: "<<step_wavelength*1e6<<" microns. Enter new wavelength factor: "<<flush;
						echo();
						getnstr(inptvalue,10);step_wavelength=strtod(inptvalue,NULL)/1e6;
						noecho();
						cout<<"\r"<<flush;
						break;
					case FOCAL_LENGTH:
						cout<<"focal length.\r\nCurrent mean focal length factor: "<<step_focal_length*1e6<<" microns. Enter new mean focal length factor: "<<flush;
						echo();
						getnstr(inptvalue,10);step_focal_length=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					case X_OFFSET:
						cout<<"x offset.\r\nCurrent x offset factor: "<<step_xoff*1e6<<" microns. Enter new x offset factor: "<<flush;
						echo();
						getnstr(inptvalue,10);step_xoff=strtod(inptvalue,NULL)/1e6;
						cout<<"\r"<<flush;
						noecho();
						break;
					default:
						cout<<"Nothing changed.\n";
						cout<<"\r"<<endl;
						break;
					}
				break;

			case QUIT:
				quit=TRUE;
				break;
			case ATTRIBUTES:
				if(first_run==TRUE){
					cout<<"Cannot determine attributes, since they have not been set yet.\r"<<endl;
					cout<<"Press `d' to set them to defaults.\r"<<endl<<endl;
					}
				else{
					cout<<"The arcs have the following characteristics:\r"<<endl;
					cout<<"Wavelength:               "<<wavelength*1e6<<" microns\r"<<endl;
					cout<<"Mean focal length:        "<<focal_length*1e6<<" microns\r"<<endl;
					cout<<"x offset:                 "<<xoff*1e6<<" microns\r"<<endl;
					cout<<"Mark:space ratio:         "<<msr<<"\r"<<endl<<endl;
					cout<<"Current scale factors:\r"<<endl;
					cout<<"Wavelength factor:        "<<step_wavelength*1e6<<" microns\r"<<endl;
					cout<<"Mean focal length factor: "<<step_focal_length*1e6<<" microns\r"<<endl;
					cout<<"x offset factor:          "<<step_xoff*1e6<<" microns\r"<<endl<<endl;
					cout<<"SLM image size:           "<<slm_height*1e6<<" microns\r"<<endl;
					cout<<"Distortion factor:        "<<distortion_factor<<"\r"<<endl;
					cout<<"Frame repetition rate:    "<<frame_rate<<"\r"<<endl<<endl;
					}
				break;
			case HELP:
				cout<<endl<<"Use these keys to adjust the arc attributes by the current scale factors:\r"<<endl;
				cout<<"   + -   increase/decrease wavelength\r"<<endl;
				cout<<"   v ^   increase/decrease mean focal length\r"<<endl;
				cout<<"   < >   move x offset to the left/right\r"<<endl<<endl;
				cout<<"To set the values absolutely, press `s' followed by:\r"<<endl;
				cout<<"    w    wavelength\r"<<endl;
				cout<<"    f    mean focal length\r"<<endl;
				cout<<"    x    x offset\r"<<endl;
				cout<<"    s    SLM size\r"<<endl;
				cout<<"    d    distortion factor\r"<<endl;
				cout<<"    m    mark:space ratio (actually the portion of light that is 'off')\r"<<endl;
				cout<<"    r    frame repetition rate\r"<<endl;
				cout<<"                     ... and then enter the value\r"<<endl<<endl;
				cout<<"To change the scale factor, press `f' followed by:\r"<<endl;
				cout<<"    w    wavelength factor\r"<<endl;
				cout<<"    f    mean focal length factor\r"<<endl;
				cout<<"    x    x offset factor\r"<<endl;
				cout<<"                     ... and then enter the value\r"<<endl<<endl;
				cout<<"Press `q' to quit, `a' to show the arc attributes, `d' for defaults.\r"<<endl;
				break;
			case DEFAULT:
				cout<<"Arc attributes, scales factors etc have been reset to default.\r"<<endl;
				cout<<"Press `a' to find out what they are now.\r"<<endl<<endl;

				xoff=0;
				focal_length=2e-3;
				wavelength=37.5e-6;
				slm_height=1.76e-3;
				distortion_factor=1.1;
				msr=0.5;
				frame_rate=1;
				step_xoff=100e-6;
				step_focal_length=0.1e-6;
				step_wavelength=1e-6;

				update_slm=TRUE;
				break;
			case RETURN:
				cout<<"\r"<<endl;
				break;
			default:
				break;
			}


		// now work out the conversion factors from real world units into SLM pixels

		py=slm_height/256; //in metres
		px=py/distortion_factor;

// Internally the compressed direction is called X and the uncompressed direction is called
// Y.  This works lovely BUT as of the physical set up of june 2001 the horizontal (distorted)
// direction is Y and the vertical (undistorted) direction is X
		if(update_slm==TRUE){
			// note that the coordinates are transposed here to match the
			// physical layout of the slm as of june 2001 matt
			// the logical slm X direction corresponds to vertical 
			// on the slm with +ve X pointing down
			// the logical Y corresponds to horizontal and is the squished aspect
			// postive Ys correspond to movement in the right hand direction
			// as determined by the camera
			SLM_upload_arcs(focal_length,xoff,py,px,wavelength,msr,frame_rate);
			update_slm=FALSE;
			first_run=FALSE;
			}		
		}while(quit!=TRUE);
		
	nocbreak(); echo(); endwin();
	}


// Internally the compressed direction is called X and the uncompressed direction is called
// Y.  This works lovely BUT as of the physical set up of june 2001 the horizontal (distorted)
// direction is Y and the vertical (undistorted) direction is X

void SLM_upload_arcs(float xoff,float yoff,float px, float py,float wavelength,float msr,int frame_rate){

int x,y;
float xc,yc,yc2;
unsigned char frame[8192];
float dist_psf[65536];
char arcs[65536];
short frl[3*2];
FILE *t;
int ap,cp,bit; //ap- arc pointer, cp- compressed (frame) pointer, bit- bit number

//cout<<"X offset="<<xoff<<"Y offset ="<<yoff<<" px="<<px<<"py="<<py<<"\n";

// calulated the distance cone to the focal spot for each pixel
// x and y are the logical coords to the arrays, xc and yc are the distances from focus to pixel at (x,y)
//cout<<"px "<<px<<" py "<<py<<"\n";
	for(y=0;y<256;y++){
		yc=((float)y-127.5)*py+yoff;
		yc2=yc*yc;// compiler should remove extra variable, leave for clarity
		for(x=0;x<256;x++){
			xc=((float)x-127.5)*px+xoff;
			dist_psf[x+(y*256)]=sqrt((xc*xc)+yc2);
			}
		}
// should go straight to frame but allow this for clarity
	for(ap=0;ap<65536;ap++){
			arcs[ap]=(char)(fmod(dist_psf[ap],wavelength) >= (wavelength*msr));
			}

// encode or compress the slm image
	for(ap=0,cp=0;cp<8192;cp++){
		frame[cp]=0;
		for(bit=1;bit<256;bit*=2,ap++){
			if(arcs[ap])frame[cp]|=bit;
			//cout<<"b "<<bit<<" ap "<<ap<<"\n";
			}
		}
	t=fopen("dist_psf.dat","w");
	fwrite((void *)dist_psf,sizeof(float),65536,t);
	fclose(t);

	t=fopen("arcs.dat","w");
	fwrite((void *)arcs,sizeof(char),65536,t);
	fclose(t);

	t=fopen("coded.dat","w");
	fwrite((void *)frame,sizeof(char),8192,t);
	fclose(t);

//	cout<<"Uploading to frame "<<1<<"\n";
// upload the data
	SLM_upload_frame(1,frame);
// halt the timer (why?)
	SLM_timer('H');
//	cout<<"Setting FRL\n";
// set the FRL to +ve frame 1, -ve frame 1
	frl[0]=1;frl[1]=0;frl[2]=1;
	frl[3]=1;frl[4]=0x8000;frl[5]=1;
// upload it
	SLM_upload_FRL(2,frl);
// set the rep rate
	SLM_timer_rate((short)frame_rate);
// set the timer to go
	SLM_timer('G');
	}



// distrotion_factor is really y_distort_sq and from the maths it is the linear stretch factor ^2 
// the array index sequence is unconventional and possibly slow
// the use of pow() rather than << is terribly slow
// y_offset is in the propagation direction (focal length)
//
// the z-axis or propagaton direction axis is the y-axis or slowly varying axis or large step axis or uncompressed axis
//
// nasty mixture of mm and microns, better off all in metres
// all SLM_ units are in pixels
//



/* original code...................
void SLM_upload_arcs(float xoff_p, float y_offset_p, float sep_p, float y_distort_sq, float msr, int frame_rate)
{
int x,y,l,b;
unsigned char fr[8192];
int frame[8192];
float cone[65536];
int arcs[65536];
short frl[3*2];
 
        for(x=0;x<256;x++){
                for(y=0;y<256;y++){
                        cone[x+(y*256)]=sqrt((y_distort_sq*(y_offset_p-y)*(y_offset_p-y))+((xoff_p-x)*(xoff_p-x)));
                        }
                }
        for(x=0;x<256;x++){
                for(y=0;y<256;y++){
                        arcs[x+(y*256)]=(fmod(cone[y+(x*256)],sep_p) >= (sep_p*msr));
                        }
                }
        for(l=0;l<8192;l++){
                frame[l]=0;
                }
 
        for(l=0;l<8192;l++){
                for(b=0;b<8;b++){
                        frame[l]=frame[l] | (int)(pow(2,b)*(arcs[(l*8)+b]!=0));
                        fr[l]=(uchar)frame[l];
                        }
                }
 
      cout<<"Uploading to frame "<<1<<"\n";
        SLM_upload_frame(1,fr);
        SLM_timer('H');
      cout<<"Setting FRL\n";
        frl[0]=1;frl[1]=0;frl[2]=1;
        frl[3]=1;frl[4]=0x8000;frl[5]=1;
        SLM_upload_FRL(2,frl);
        SLM_timer_rate((short)frame_rate);
        SLM_timer('G');
        }      
*/
