/***
*	$Id: par_port.cpp 2.2.1.1 1997/04/11 16:21:10 DavidB Development David_Banas $
*
*	Copyright (c) 1996 by Displaytech, Inc.  2200 Central Ave.  Boulder, CO
*
*	$Revision: 2.2.1.1 $
*
*	$Name: Rel_1 $
*
*	Revision History:
*
*	$Log: par_port.cpp $
*	Revision 2.2.1.1  1997/04/11 16:21:10  DavidB
*	Fixed some type casting bugs
*	Revision 2.2  1997/03/27 23:18:40  DavidB
*	Modified status checking to check SLM presence & Vwin
*	Revision 2.1  1996/12/26 23:58:08  DavidB
*	Initial revision
***/

#include <stdio.h>
#include <stdlib.h>
#include "port_stuff.h"
#include "portdefs.h"
#include <ctype.h>
#include "winslm.h"
#include <iostream.h>

#define bytesPerPlane		8192
#define bytesPerVRAMRow		2048
#define VRAMRowsPerPlane	4
#define bytesPerBlock		6
#define blocksPerVRAMRow	341
#define remainingBytes		2
#define MCUTimerRate		125000

// Register Select Bit definitions
#define REG0_SEL		(((0x00^5)<<1)&0xE)
#define REG1_SEL		(((0x01^5)<<1)&0xE)
#define REG2_SEL		(((0x02^5)<<1)&0xE)
#define REG3_SEL		(((0x03^5)<<1)&0xE)
#define REG4_SEL		(((0x04^5)<<1)&0xE)
#define REG5_SEL		(((0x05^5)<<1)&0xE)
#define REG6_SEL		(((0x06^5)<<1)&0xE)
#define REG7_SEL		(((0x07^5)<<1)&0xE)
// Parasite Status Registers
#define NUM_REGS		9
#define STAT_SYS		0
#define STAT_CMD		1
#define STAT_VRM		2
#define STAT_FRL		3
#define STAT_PRK		4
#define STAT_COP		5
#define STAT_CLK		6
#define STAT_SLM		7
#define STAT_WIN		8

#define BUSY()			((inportb(LPT_STAT) & 0x80) == 0)
#define CMD_DONE		0x20
#define INVLD_CMD		0x08
#define INVLD_PARAM		0x10

#define dummyCmd		0
#define maxTries		65535	// Parallel Port Busy Timeout

struct ppBlock
{
	char		cmd;
    char		numParams;
	short		param0;
    short		param1;
    short		param2;
};

bool	isChanelBusy( void ) throw();
errCode sendCmd( ppBlock &ppCmd ) throw();
errCode uploadPlane( unsigned short plane, unsigned char *data ) throw();
errCode outputFRL( unsigned short count, short *data ) throw();
void 	myOutp( unsigned char reg, unsigned char data ) throw();
errCode	myWait( void ) throw();


// refresh
void
refresh( void ) throw()
{
	ppBlock		ppCmd;

    ppCmd.cmd = 'S';					// Execute stat. cmd to update LEDs.
    ppCmd.numParams = 1;
    ppCmd.param0 = STAT_SYS;
    sendCmd( ppCmd );
}


// getStatus
errCode
getStatus( statBlock &status ) throw()
{
	int			regNum;
	char		statReg[NUM_REGS];
	errCode		theErr;
	ppBlock		ppCmd;

    ppCmd.cmd = 'S';				// 'Status' command
    ppCmd.numParams = 1;			// 1 parameter needed for 'status' command
    for( regNum = 0; regNum < NUM_REGS; regNum++ )
    {
    	ppCmd.param0 = regNum;		// Get status reg. #(regNum).
        if( (theErr = sendCmd( ppCmd )) != EC_NO_ERROR )
        	return theErr;
    	// Store the status register's value.
		statReg[regNum] = inportb(LPT_STAT);
	cout<<128+(int)statReg[regNum]<<":";
	}
	cout<<endl;
    // Restore the front panel LEDs.
	ppCmd.param0 = STAT_SYS;
    if( (theErr = sendCmd( ppCmd )) != EC_NO_ERROR )
    	return theErr;

    // System Status Register
	if( statReg[STAT_SYS] & 0x08 )		status.sys = SYS_FAULT;
    else if( statReg[STAT_SYS] & 0x10 )	status.sys = SYS_PARKED;
    else								status.sys = SYS_NORMAL;

    // Command Status Register
	if( statReg[STAT_CMD] & 0x08 )		status.cmd = CMD_BAD_CMD;
    else if( statReg[STAT_CMD] & 0x10 )	status.cmd = CMD_BAD_PARAM;
    else								status.cmd = CMD_GOOD;

    // FRL Status Register
	if( statReg[STAT_FRL] & 0x08 )		status.frl = FRL_BAD_PTR;
    else if( statReg[STAT_FRL] & 0x10 )	status.frl = FRL_BAD_PAGE;
    else								status.frl = FRL_GOOD;

    // SLM & Vwin Status Register
    if( statReg[STAT_SLM] & 0x20 )		// if SLM 1 is plugged in
	{
    	if( statReg[STAT_PRK] & 0x08 )		status.slm1 = SLM_PARKED;
    	else								status.slm1 = SLM_RUNNING;

        if( statReg[STAT_WIN] & 0x20 )		status.win1 = WIN_BAD;
        else								status.win1 = WIN_GOOD;
    }
    else
    {
    	status.slm1 = SLM_MISSING;
        status.win1 = WIN_NA;
    }

    if( statReg[STAT_SLM] & 0x10 )		// if SLM 2 is plugged in
    {
    	if( statReg[STAT_PRK] & 0x10 )		status.slm2 = SLM_PARKED;
    	else								status.slm2 = SLM_RUNNING;

        if( statReg[STAT_WIN] & 0x10 )		status.win2 = WIN_BAD;
        else								status.win2 = WIN_GOOD;
	}
    else
    {
    	status.slm2 = SLM_MISSING;
		status.win2 = WIN_NA;
    }
    
    // COP Status Register
    status.copRsts = 0;
	if( statReg[STAT_COP] & 0x08 )		status.copRsts += 4;
	if( statReg[STAT_COP] & 0x10 )		status.copRsts += 2;
	if( statReg[STAT_COP] & 0x20 )		status.copRsts += 1;

    // CLK Status Register
    status.clkRsts = 0;
	if( statReg[STAT_CLK] & 0x08 )		status.clkRsts += 4;
	if( statReg[STAT_CLK] & 0x10 )		status.clkRsts += 2;
	if( statReg[STAT_CLK] & 0x20 )		status.clkRsts += 1;

    return EC_NO_ERROR;
}

// doCmd
errCode
doCmd( cmdBlock &theCmd ) throw()
{
	ppBlock		ppCmd;
	errCode		theErr;

	switch( theCmd.cmd )
    {
        case CMD_DUMMY:
			ppCmd.cmd = 0;
            ppCmd.numParams = 0;
            break;

        case CMD_RESET:
        	ppCmd.cmd = 'R';
        	ppCmd.numParams = 0;
            break;

		case CMD_TIMER:
        	ppCmd.cmd = 'T';
        	switch( theCmd.subCmd )
            {
            	case TMR_STOP:
					ppCmd.numParams = 'H';
                    break;

                case TMR_GO:
					ppCmd.numParams = 'G';
                    break;

                case TMR_STEP:
					ppCmd.numParams = 'S';
                    break;

                case TMR_RATE:
					ppCmd.numParams = 'R';
            		ppCmd.param0 = MCUTimerRate / theCmd.cnt;
                    break;

                default:
                	return EC_INVALID_CMD;
            }
            break;

		case CMD_UPLOAD:
			return( uploadPlane( theCmd.cnt, (unsigned char *)theCmd.ptr ) );

        case CMD_OUTPUT:
        	return( outputFRL( theCmd.cnt, (short *)theCmd.ptr ) );

        case CMD_ENTRY:
        	ppCmd.cmd = 'E';
            ppCmd.numParams = 1;
            ppCmd.param0 = theCmd.cnt;
            break;

        case CMD_REPEAT:
        	ppCmd.cmd = 'F';
        	ppCmd.numParams = 1;
        	ppCmd.param0 = theCmd.cnt;
            break;

        case CMD_FIDUCIALS:
        	ppCmd.cmd = 'D';
            ppCmd.numParams = 2;
            ppCmd.param0 = theCmd.subCmd;
            ppCmd.param1 = theCmd.cnt;
			break;

        case CMD_APRON:
        	ppCmd.cmd = 'A';
            ppCmd.numParams = 2;
            ppCmd.param0 = theCmd.subCmd;
            ppCmd.param1 = theCmd.cnt;
			break;

        case CMD_MASK:
        	ppCmd.cmd = 'M';
            ppCmd.numParams = 2;
            ppCmd.param0 = theCmd.subCmd;
            ppCmd.param1 = theCmd.cnt;
			break;

        default:
        	return EC_INVALID_CMD;
	}
    return( sendCmd( ppCmd ) );
}


// uploadPlane
errCode
uploadPlane( unsigned short plane, unsigned char *data ) throw()
{
	int			row, offset, block;
	ppBlock		ppCmd;
	errCode		theErr;

    for( row = 0; row < VRAMRowsPerPlane; row++ )
    {
    	ppCmd.cmd = 'U';
    	ppCmd.numParams = 2;
    	ppCmd.param0 = plane * VRAMRowsPerPlane + row;	// VRAM row
    	ppCmd.param1 = 0;								// VRAM offset
	cout<<"ppCmd.param0 "<<ppCmd.param0<<endl;
    	if( (theErr = sendCmd( ppCmd )) != EC_NO_ERROR )
    		return( theErr );
        ppCmd.cmd = 'V';			// Upload the data
        ppCmd.numParams = 3;				// in 6-byte chuncks.
    	for( block = 0; block < blocksPerVRAMRow; block++ )
        {
        	offset = row*bytesPerVRAMRow + block*bytesPerBlock;
			ppCmd.param0 = (data[offset] << 8) + data[offset+1];
			ppCmd.param1 = (data[offset+2] << 8) + data[offset+3];
			ppCmd.param2 = (data[offset+4] << 8) + data[offset+5];
			if( (theErr = sendCmd( ppCmd )) != EC_NO_ERROR )
            	return( theErr );
        }
        ppCmd.numParams = 1;				// We have 2 more bytes to send.
        offset = row*bytesPerVRAMRow + block*bytesPerBlock;
        ppCmd.param0 = (data[offset] << 8) + data[offset+1];
        if( (theErr = sendCmd( ppCmd )) != EC_NO_ERROR )
        	return( theErr );
    }
    return( EC_NO_ERROR );
}


// outputFRL
errCode
outputFRL( unsigned short count, short *data ) throw()
{
	int			i,j=0;
	ppBlock		ppCmd;
    errCode		theErr;

    ppCmd.cmd = 'O';
    ppCmd.numParams = 'W';					// 'Write' the 1st entry
    ppCmd.param0 = data[0];
    ppCmd.param1 = data[1];
    ppCmd.param2 = data[2];
    if( (theErr = sendCmd( ppCmd )) != EC_NO_ERROR )
    	return( theErr );
	cout<<j++<<"\n";
    ppCmd.numParams = 'A';					// and 'Append' the rest.
    for( i = 1; i < count; i++ )
    {
        ppCmd.param0 = data[i*3];
        ppCmd.param1 = data[i*3 + 1];
        ppCmd.param2 = data[i*3 + 2];
		if( (theErr = sendCmd( ppCmd )) != EC_NO_ERROR )
        	return( theErr );
	cout<<j++<<"\n";
    }
    return( EC_NO_ERROR );
}


// Send the command over the parallel port and check status
bool
isChanelBusy( void ) throw()
{
	if BUSY()
    	return true;
    else
    	return false;
}


errCode
sendCmd( ppBlock &ppCmd ) throw()
{
	errCode		theErr;
	char		cmdStatus;

    if( (theErr = myWait()) != EC_NO_ERROR )	// wait till all 8 registers
    	return( theErr );			// have been fetched or timeout occurs.

	// Output command to Parasite
	myOutp( REG1_SEL, ppCmd.numParams );	// Load FPGA registers #1 - 7 first
    switch( ppCmd.numParams )				// Send only the required Parameters.
    {
    	default:
    		myOutp( REG6_SEL, (char)(ppCmd.param2 >> 8) );
    		myOutp( REG7_SEL, (char)(ppCmd.param2 & 0x00FF) );

        case 2:
			myOutp( REG4_SEL, (char)(ppCmd.param1 >> 8) );
    		myOutp( REG5_SEL, (char)(ppCmd.param1 & 0x00FF) );

		case 1:
    		myOutp( REG2_SEL, (char)(ppCmd.param0 >> 8) );
    		myOutp( REG3_SEL, (char)(ppCmd.param0 & 0x00FF) );

        case 0:
        	break;
    }
    myOutp( REG0_SEL, ppCmd.cmd );	// Loading FPGA reg. #0 triggers int.
    if( (theErr = myWait()) != EC_NO_ERROR )	// wait till all 8 registers
    	return( theErr );		// have been fetched or timeout occurs.

    if( ppCmd.cmd == 'S' )		// If this was a 'status' command,
	{
    	// Output a dummy cmd to insure processing of the previous cmd.
		myOutp( REG1_SEL, 0 );
		myOutp( REG0_SEL, dummyCmd );
    	if( (theErr = myWait()) != EC_NO_ERROR )	// wait till command
    		return( theErr );		// has been fetched or timeout occurs.
        return EC_NO_ERROR;		// return now.
	}

    // check command status.
    myOutp( REG1_SEL, 1 );
    myOutp( REG3_SEL, STAT_CMD );
    myOutp( REG0_SEL, 'S' );		// 'S' is the "Update Status Bits" cmd.
    if( (theErr = myWait()) != EC_NO_ERROR )	// wait till command
    	return( theErr );		// has been fetched or timeout occurs.

    // Output a dummy cmd to insure processing of the previous cmd.
	myOutp( REG1_SEL, 0 );
	myOutp( REG0_SEL, dummyCmd );
    if( (theErr = myWait()) != EC_NO_ERROR )	// wait till command
    	return( theErr );		// has been fetched or timeout occurs.

    // Store the command status register's value.
	cmdStatus = inportb(LPT_STAT);

    // Restore front panel LEDs to system status register state.
    myOutp( REG1_SEL, 1 );
    myOutp( REG3_SEL, STAT_SYS );
    myOutp( REG0_SEL, 'S' );		// 'S' is the "Update Status Bits" cmd.

    // Return the command status register's state.
    if( cmdStatus & CMD_DONE )
    	return( EC_NO_ERROR );
    else if( cmdStatus & INVLD_CMD )
    	return( EC_INVALID_CMD );
    else if( cmdStatus & INVLD_PARAM )
    	return( EC_INVALID_PARAM );
    else
    	return( EC_TIMEOUT );
}


// myOutp
void
myOutp( unsigned char reg, unsigned char data ) throw()
{
	int			i;

//	for( i = 0; i < 1000; i++ );
    outportb( LPT_CTRL, reg );			// Setup register select lines.
cout<<"("<<(int)reg<<",";
//	for( i = 0; i < 1000; i++ );
    outportb( LPT_DATA, data );			// Setup the data.
cout<<(int)data<<")";
//	for( i = 0; i < 1000; i++ );
    outportb( LPT_CTRL, reg | 1 );		// Toggle nSTROBE.
//	for( i = 0; i < 1000; i++ );
    outportb( LPT_CTRL, reg );
}


// myWait
errCode
myWait( void ) throw()
{
	unsigned long		i=maxTries;

    while( BUSY() )
    	if( --i <= 0 )
        	return EC_TIMEOUT;

    return EC_NO_ERROR;
}

